/*
	*
	* Copyright 2019-2021 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* at your option, any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

// Local Headers
#include "DesQMainWindow.hpp"
#include "DesQActionBar.hpp"

DesQMainWindow::DesQMainWindow() : QMainWindow() {

	/* Mobile interface: Start as false */
	mobileUI = false;

	/* Install the event filter */
	createUI();

	/* Install the event filter to catch the titlebar double click */
	titleBar->installEventFilter( this );
};

void DesQMainWindow::createUI() {

	/* TitleBar */
	titleBar = new QWidget();
	titleBar->setObjectName( "TitleBar" );
	titleBar->setFixedHeight( 32 );

	QPalette tPlt( palette() );
	tPlt.setColor( QPalette::Window, tPlt.color( QPalette::Window ).darker( 110 ) );
	titleBar->setAutoFillBackground( true );
	titleBar->setPalette( tPlt );

	/* App Icon */
	if ( mobileUI ) {
		iconBtn = new QToolButton();
		iconBtn->setIcon( QIcon::fromTheme( "desq" ) );
		iconBtn->setIconSize( QSize( 24, 24 ) );
		iconBtn->setFixedSize( QSize( 32, 32 ) );
		iconBtn->setFocusPolicy( Qt::NoFocus );
		iconBtn->setAutoRaise( true );
		connect( iconBtn, &QToolButton::clicked, this, &DesQMainWindow::toggleActionBar );
	}

	else {
		iconLbl = new QLabel();
		iconLbl->setFixedSize( QSize( 32, 32 ) );
		iconLbl->setPixmap( QIcon::fromTheme( "desq" ).pixmap( 24 ) );
		iconLbl->setAlignment( Qt::AlignCenter );

		titleBar->hide();
	}

	/* App Label */
	titleLbl = new QLabel( "DesQ Main Window" );
	titleLbl->setFont( QFont( font().family(), 10, QFont::Bold ) );

	/* Close Button */
	quitBtn = new QToolButton();
	quitBtn->setIcon( QIcon::fromTheme( "window-close" ) );
	quitBtn->setIconSize( QSize( 24, 24 ) );
	quitBtn->setFixedSize( QSize( 32, 32 ) );
	quitBtn->setFocusPolicy( Qt::NoFocus );
	quitBtn->setAutoRaise( true );
	connect( quitBtn, &QToolButton::clicked, this, &DesQMainWindow::close );

	QHBoxLayout *titleLyt = new QHBoxLayout();
	titleLyt->setContentsMargins( QMargins() );
	titleLyt->setSpacing( 0 );

	if ( mobileUI )
		titleLyt->addWidget( iconBtn );

	else
		titleLyt->addWidget( iconLbl );

	titleLyt->addStretch();
	titleLyt->addWidget( titleLbl );
	titleLyt->addStretch();
	titleLyt->addWidget( quitBtn );
	titleBar->setLayout( titleLyt );

	/* Main View Dummy */
	mainView = new QWidget();
	mainView->setMinimumSize( QSize( 300, 300 ) );
	mainView->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ) );

	/* ActionBar */
	actionBar = new DesQActionBar( this );
	actionBar->setObjectName( "ActionBar" );
	actionBar->setFixedHeight( 44 );
	connect( actionBar, &DesQActionBar::action, this, &DesQMainWindow::action );

	QPalette aPlt( palette() );
	aPlt.setColor( QPalette::Window, aPlt.color( QPalette::Base ) );
	actionBar->setAutoFillBackground( true );
	actionBar->setPalette( aPlt );

	/* Main Layout */
	mainLyt = new QGridLayout();
	mainLyt->setContentsMargins( QMargins() );
	mainLyt->setSpacing( 0 );

	mainLyt->addWidget( titleBar, 0, 0 );
	mainLyt->addWidget( mainView, 1, 0 );
	mainLyt->addWidget( actionBar, 2, 0 );

	QWidget *base = new QWidget();
	base->setLayout( mainLyt );
	setCentralWidget( base );
};

/* Main Window */
void DesQMainWindow::setMainView( QWidget *view ) {

	mainLyt->removeWidget( mainView );
	delete mainView;

	mainView = view;
	mainLyt->addWidget( mainView, 1, 0 );
};

/* Application Title */
void DesQMainWindow::setAppTitle( QString title ) {

	setWindowTitle( title );
	titleLbl->setText( title );
};

/* Application icon */
void DesQMainWindow::setAppIcon( QString icoStr ) {

	if ( mobileUI )
		iconBtn->setIcon( QIcon::fromTheme( icoStr, QIcon( icoStr ) ) );

	else
		iconLbl->setPixmap( QIcon::fromTheme( icoStr, QIcon( icoStr ) ).pixmap( 24 ) );
};

void DesQMainWindow::setAppIcon( QIcon icon ) {

	if ( mobileUI )
		iconBtn->setIcon( icon );

	else
		iconLbl->setPixmap( icon.pixmap( 24 ) );
};

/* Add items to titlebar */
void DesQMainWindow::addTitleBarWidget( QWidget *widget, int pos ) {

	Q_UNUSED( widget );		// -> Widget to be added to the title bar
	Q_UNUSED( pos );		// -> Position to add the widget (0 = left, next to the app icon and 1 = right, next to the close button)
};

/* Add floating action to the main view */
void DesQMainWindow::setFloatingAction( QString name, QIcon icon, QString target ) {

	/* If the floating widget exists, delete it */
	if ( mFloatingWidget ) {
		/* Disconnect thall the signals-slot connections */
		mFloatingWidget->disconnect();

		/* Remove it from the layout */
		mainLyt->removeWidget( mFloatingWidget );

		/* Delete the widget */
		mFloatingWidget->deleteLater();
	}

	QToolButton *btn = new QToolButton();
	btn->setAutoRaise( true );

	QPalette btnPlt = palette();
	btnPlt.setColor( QPalette::Button, QColor( Qt::transparent ) );
	btn->setPalette( btnPlt );

	btn->setIcon( icon );
	btn->setToolTip( name );
	btn->setIconSize( QSize( 64, 64 ) );
	btn->setFixedSize( QSize( 72, 72 ) );

	/* Create the connection */
	connect( btn, &QToolButton::clicked, [=](){ emit action( target ); } );

	/* Cast the button into a widget and add it to the layout */
	mFloatingWidget = qobject_cast<QWidget *>( btn );
	mainLyt->addWidget( mFloatingWidget, 1, 0, Qt::AlignRight| Qt::AlignBottom );
};

/* Add floating action to the main view */
void DesQMainWindow::setFloatingWidget( QWidget *widget ) {

	/* If the floating widget exists, delete it */
	if ( mFloatingWidget ) {
		/* Disconnect thall the signals-slot connections */
		mFloatingWidget->disconnect();

		/* Remove it from the layout */
		mainLyt->removeWidget( mFloatingWidget );

		/* Delete the widget */
		mFloatingWidget->deleteLater();
	}

	/* Save the widget into mFloatingWidget and add it to the layout */
	mFloatingWidget = widget;
	mainLyt->addWidget( mFloatingWidget, 1, 0, Qt::AlignRight| Qt::AlignBottom );
};

/* Add items to actionBar */
int DesQMainWindow::addAction( int page, QString name, QIcon icon, QString target, QString tooltip, bool checkable, bool checked ) {

	return actionBar->addAction( page, name, icon, target, tooltip, checkable, checked );
};

/* Add space/stretch to actionBar */
void DesQMainWindow::addSpace( int page, bool stretch ) {

	if ( stretch )
		actionBar->addStretch( page );

	else
		actionBar->addSingleSpace( page );
};

void DesQMainWindow::addSeparator( int page ) {

	actionBar->addSeparator( page );
};

void DesQMainWindow::setIcon( int page, int item, QIcon icon ) {

	actionBar->setIcon( page, item, icon );
};

void DesQMainWindow::setActionEnabled( int page, int item ) {

	actionBar->setItemEnabled( page, item );
};

void DesQMainWindow::setActionDisabled( int page, int item ) {

	actionBar->setItemDisabled( page, item );
};

/* Even filtering to get title bar double press */
bool DesQMainWindow::eventFilter( QObject *obj, QEvent *event ) {

	/* Object based */
	if ( obj == titleBar ) {
		if ( event->type() == QEvent::MouseButtonDblClick ) {
			tryMaximize();
			return true;
		}
	}

	/* Event-type based */
	if ( event->type() == QEvent::WindowActivate ) {
		QPalette tPlt( palette() );
		tPlt.setColor( QPalette::Window, tPlt.color( QPalette::Window ).darker( 110 ) );
		titleBar->setAutoFillBackground( true );
		titleBar->setPalette( tPlt );

		return true;
	}

	else if ( event->type() == QEvent::WindowDeactivate ) {
		QPalette tPlt( palette() );
		tPlt.setColor( QPalette::Window, tPlt.color( QPalette::Text ).darker( 120 ) );
		titleBar->setAutoFillBackground( true );
		titleBar->setPalette( tPlt );

		return true;
	}

	return QMainWindow::eventFilter( obj, event );
};

/* This is MobileUI special */
void DesQMainWindow::toggleActionBar() {

	/* If the actionBar is visible, hide it and show the mainView */
	if ( actionBar->isVisible() )
		actionBar->hide();

	/* Otherwise, show the actionBar and hide the mainView */
	else
		actionBar->show();
};

void DesQMainWindow::tryMaximize() {

	/* If we are in mobileUI mode, we will be maximized. So, nothing doing. */
	if ( mobileUI )
		return;

	if ( isMaximized() )
		showNormal();

	else
		showMaximized();
};
